/*
 * @adonisjs/http-server
 *
 * (c) AdonisJS
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import is from '@sindresorhus/is';
import { parseRange } from './helpers.js';
import * as errors from './exceptions.js';
/**
 * The base HTTP exception handler one can inherit from to handle
 * HTTP exceptions.
 *
 * The HTTP exception handler has support for
 *
 * - Ability to render exceptions by calling the render method on the exception.
 * - Rendering status pages
 * - Pretty printing errors during development
 * - Transforming errors to JSON or HTML using content negotiation
 * - Reporting errors
 */
export class ExceptionHandler {
    /**
     * Whether or not to render debug info. When set to true, the errors
     * will have the complete error stack.
     */
    debug = process.env.NODE_ENV !== 'production';
    /**
     * Whether or not to render status pages. When set to true, the unhandled
     * errors with matching status codes will be rendered using a status
     * page.
     */
    renderStatusPages = process.env.NODE_ENV === 'production';
    /**
     * A collection of error status code range and the view to render.
     */
    statusPages = {};
    /**
     * Computed from the status pages property
     */
    #expandedStatusPages;
    /**
     * Renderers for rendering an error.
     */
    renderers = {
        html: async (error, ctx) => {
            if (this.isDebuggingEnabled(ctx)) {
                const { default: Youch } = await import('youch');
                const html = await new Youch(error, ctx.request.request).toHTML();
                ctx.response.status(error.status).send(html);
                return;
            }
            ctx.response.status(error.status).send(`<p> ${error.message} </p>`);
        },
        json: async (error, ctx) => {
            if (this.isDebuggingEnabled(ctx)) {
                const { default: Youch } = await import('youch');
                const json = await new Youch(error, ctx.request.request).toJSON();
                ctx.response.status(error.status).send(json.error);
                return;
            }
            ctx.response.status(error.status).send({ message: error.message });
        },
        json_api: async (error, ctx) => {
            if (this.isDebuggingEnabled(ctx)) {
                const { default: Youch } = await import('youch');
                const json = await new Youch(error, ctx.request.request).toJSON();
                ctx.response.status(error.status).send(json.error);
                return;
            }
            ctx.response.status(error.status).send({
                errors: [
                    {
                        title: error.message,
                        code: error.code,
                        status: error.status,
                    },
                ],
            });
        },
    };
    /**
     * Enable/disable errors reporting
     */
    reportErrors = true;
    /**
     * An array of exception classes to ignore when
     * reporting an error
     */
    ignoreExceptions = [
        errors.E_HTTP_EXCEPTION,
        errors.E_ROUTE_NOT_FOUND,
        errors.E_CANNOT_LOOKUP_ROUTE,
        errors.E_HTTP_REQUEST_ABORTED,
    ];
    /**
     * An array of HTTP status codes to ignore when reporting
     * an error
     */
    ignoreStatuses = [400, 422, 401];
    /**
     * An array of error codes to ignore when reporting
     * an error
     */
    ignoreCodes = [];
    /**
     * Expands status pages
     */
    #expandStatusPages() {
        if (!this.#expandedStatusPages) {
            this.#expandedStatusPages = Object.keys(this.statusPages).reduce((result, range) => {
                const renderer = this.statusPages[range];
                result = Object.assign(result, parseRange(range, renderer));
                return result;
            }, {});
        }
        return this.#expandedStatusPages;
    }
    /**
     * Forcefully tweaking the type of the error object to
     * have known properties.
     */
    #toHttpError(error) {
        const httpError = is.object(error) ? error : new Error(String(error));
        httpError.message = httpError.message || 'Internal server error';
        httpError.status = httpError.status || 500;
        return httpError;
    }
    /**
     * Error reporting context
     */
    context(ctx) {
        const requestId = ctx.request.id();
        return requestId
            ? {
                'x-request-id': requestId,
            }
            : {};
    }
    /**
     * The format in which the error should be rendered.
     */
    getResponseFormat(ctx) {
        switch (ctx.request.accepts(['html', 'application/vnd.api+json', 'json'])) {
            case 'application/vnd.api+json':
                return 'json_api';
            case 'json':
                return 'json';
            case 'html':
            default:
                return 'html';
        }
    }
    /**
     * Returns the log level for an error based upon the error
     * status code.
     */
    getErrorLogLevel(error) {
        if (error.status >= 500) {
            return 'error';
        }
        if (error.status >= 400) {
            return 'warn';
        }
        return 'info';
    }
    /**
     * A boolean to control if errors should be rendered with
     * all the available debugging info.
     */
    isDebuggingEnabled(_) {
        return this.debug;
    }
    /**
     * Returns a boolean by checking if an error should be reported.
     */
    shouldReport(error) {
        if (this.reportErrors === false) {
            return false;
        }
        if (this.ignoreStatuses.includes(error.status)) {
            return false;
        }
        if (error.code && this.ignoreCodes.includes(error.code)) {
            return false;
        }
        if (this.ignoreExceptions.find((exception) => error instanceof exception)) {
            return false;
        }
        return true;
    }
    /**
     * Reports an error during an HTTP request
     */
    async report(error, ctx) {
        const httpError = this.#toHttpError(error);
        if (!this.shouldReport(httpError)) {
            return;
        }
        if (typeof httpError.report === 'function') {
            httpError.report(httpError, ctx);
            return;
        }
        /**
         * Log the error using the logger
         */
        const level = this.getErrorLogLevel(httpError);
        ctx.logger.log(level, {
            ...(level === 'error' || level === 'fatal' ? { err: httpError } : {}),
            ...this.context(ctx),
        }, httpError.message);
    }
    /**
     * Handles the error during the HTTP request.
     */
    async handle(error, ctx) {
        const httpError = this.#toHttpError(error);
        /**
         * Self handle exception
         */
        if (typeof httpError.handle === 'function') {
            return httpError.handle(httpError, ctx);
        }
        /**
         * Render status page
         */
        const statusPages = this.#expandStatusPages();
        if (this.renderStatusPages && statusPages[httpError.status]) {
            return statusPages[httpError.status](httpError, ctx);
        }
        /**
         * Use the format renderers.
         */
        const responseFormat = this.getResponseFormat(ctx);
        return this.renderers[responseFormat](httpError, ctx);
    }
}
